local f = WINDOW_MANAGER:CreateTopLevelWindow("ZAM_ReactionColors")
local reticle = ZO_ReticleContainerReticle
local stealthEye = ZO_ReticleContainerStealthEye
local GetUnitReactionColor = GetUnitReactionColor
local DoesUnitExist = DoesUnitExist
local GetUnitLevel = GetUnitLevel
local GetUnitReaction = GetUnitReaction
local GetConColor = GetConColor
local playerLevel = GetUnitLevel("player")
local reaction, targetName, targetLevel, warning, lowHealth, db
local colorByChoices = {"Reaction", "Level", "Level if hostile", "Disabled"}

local defaults = {
	warning = true,
	warningLevel = 5,
	lowHealth = false,
	healthLevel = 25,
	colorBy = "Reaction",
	colorTargetName = true,
	colorTargetLevel = true,
}
	

local function CreateWarningTexture()
	warning = WINDOW_MANAGER:CreateControl("ReactionColorsReticleWarning", ZO_ReticleContainer, CT_TEXTURE)
	warning:SetTexture("ZAM_ReactionColors\\levelWarning.dds")
	warning:SetDimensions(130,130)
	warning:SetAnchor(CENTER, ZO_ReticleContainer, CENTER, 0, 0)
	warning:SetColor(1, 0, 0, 1)
	warning:SetHidden(true)
end

local function CreateLowHealthTexture()
	lowHealth = WINDOW_MANAGER:CreateControl("ReactionColorsReticleLowHealth", ZO_ReticleContainer, CT_TEXTURE)
	lowHealth:SetTexture("ZAM_ReactionColors\\targetLowHealth.dds")
	lowHealth:SetDimensions(130,130)
	lowHealth:SetAnchor(CENTER, ZO_ReticleContainer, CENTER, 0, 0)
	lowHealth:SetColor(1, 0, 0, 1)
	lowHealth:SetHidden(true)
end

local function UpdateColor()
	if DoesUnitExist("reticleover") then
		local r, g, b = GetUnitReactionColor("reticleover")
		local level = GetUnitLevel("reticleover")
		local r2, g2, b2 = GetConColor(level, playerLevel)
		local hostile = GetUnitReaction("reticleover") == UNIT_REACTION_HOSTILE
		if db.colorBy == "Reaction" then
			reticle:SetColor(r, g, b)
			stealthEye:SetColor(r, g, b)
		elseif db.colorBy == "Level" or (db.colorBy == "Level if hostile" and hostile) then
			reticle:SetColor(r2, g2, b2)
			stealthEye:SetColor(r2, g2, b2)
		--[[elseif db.colorBy == "Level if hostile" and hostile then
			reticle:SetColor(r2, g2, b2)
			stealthEye:SetColor(r2, g2, b2)]]
		end
		if db.colorTargetName then
			if not targetName then targetName = ZO_TargetUnitFramereticleoverName end
			targetName:SetColor(r, g, b, 1)
		end
		if db.colorTargetLevel then
			if not targetLevel then targetLevel = ZO_TargetUnitFramereticleoverLevel end
			targetLevel:SetColor(r2, g2, b2)
		end
		if warning and db.warning then
			if hostile and (level - playerLevel >= db.warningLevel) then
				warning:SetHidden(false)
			end
		end
		if lowHealth and db.lowHealth and not hostile and not lowHealth:IsHidden() then
			lowHealth:SetHidden(true)
		end
	else
		warning:SetHidden(true)
		reticle:SetColor(1, 1, 1, 1)
		stealthEye:SetColor(1, 1, 1, 1)
		if lowHealth and db.lowHealth then lowHealth:SetHidden(true) end
	end
end

local function CreateOptions()
	local LAM = LibStub("LibAddonMenu-1.0")
	local zamPanel = LAM:CreateControlPanel("ZAM_ADDON_OPTIONS", "ZAM Addons")
	LAM:AddHeader(zamPanel, "ZAM_ReactionColors_Options_Header", "ZAM ReactionColors")
	LAM:AddDropdown(zamPanel, "ZAM_ReactionColors_Options_ColorBy", "Color the targeting reticle by...",
					"Select when to color the targeting reticle.", colorByChoices,
					function() return db.colorBy end,
					function(colorBy)db.colorBy = colorBy end)
	LAM:AddCheckbox(zamPanel, "ZAM_ReactionColors_Options_Warning", "Warning Texture", "Display a warning texture for high level hostile targets.",
					function() return db.warning end,
					function()
						db.warning = not db.warning
						if db.warning and not warning then CreateWarningTexture() end
					end)
	LAM:AddSlider(zamPanel, "ZAM_ReactionColors_Options_WarningLevel", "Warning Threshold",
					"Display the warning icon if the target is this many levels higher than you.", 2, 10, 1,
					function() return db.warningLevel end,
					function(value) db.warningLevel = value end)
	LAM:AddCheckbox(zamPanel, "ZAM_ReactionColors_Options_LowHealth", "Low Health Texture", "Display a texture when your target's health gets low.",
					function() return db.lowHealth end,
					function()
						db.lowHealth = not db.lowHealth
						if db.lowHealth and not lowHealth then CreateLowHealthTexture() end
					end)
	LAM:AddSlider(zamPanel, "ZAM_ReactionColors_Options_HealthLevel", "Health Threshold",
					"Display the low health texture if the target is at this health percentage.", 5, 95, 5,
					function() return db.healthLevel end,
					function(value) db.healthLevel = value end)
	LAM:AddCheckbox(zamPanel, "ZAM_ReactionColors_Options_ColorTargetName", "Color Target's Name", "Color the target's name by reaction.",
					function() return db.colorTargetName end,
					function()
						db.colorTargetName = not db.colorTargetName
						if not db.colorTargetName then
							targetName:SetColor(1, 1, 1)
						end
					end)
	LAM:AddCheckbox(zamPanel, "ZAM_ReactionColors_Options_ColorTargetLevel", "Color Target's Level", "Color the target's level by difficulty.",
					function() return db.colorTargetLevel end,
					function()
						db.colorTargetLevel = not db.colorTargetLevel
						if not db.colorTargetLevel then
							targetLevel:SetColor(1, 1, 1)
						end
					end)
end

local function Initialize()
	ZAM_ReactionColorsDB = ZAM_ReactionColorsDB or {}
	for k,v in pairs(defaults) do
	    if type(ZAM_ReactionColorsDB[k]) == "nil" then
	        ZAM_ReactionColorsDB[k] = v
	    end
	end
	db = ZAM_ReactionColorsDB

	if db.lowHealth then
		CreateLowHealthTexture()
	end
	if db.warning then
		CreateWarningTexture()
	end

	CreateOptions()
end
	

EVENT_MANAGER:RegisterForEvent("ZAM_ReactionColors", EVENT_RETICLE_TARGET_CHANGED, UpdateColor)
EVENT_MANAGER:RegisterForEvent("ZAM_ReactionColors", EVENT_MOUSEOVER_CHANGED, UpdateColor)
EVENT_MANAGER:RegisterForEvent("ZAM_ReactionColors", EVENT_DISPOSITION_UPDATE, UpdateColor)
EVENT_MANAGER:RegisterForEvent("ZAM_ReactionColors", EVENT_POWER_UPDATE, function(event, unitTag, powerIndex, powerType, powerValue, powerMax, powerEffectiveMax)
		if unitTag == "reticleover" and db.lowHealth and powerType == POWERTYPE_HEALTH then
			lowHealth:SetHidden(powerValue/powerEffectiveMax*100 > db.healthLevel)
		end
	end)
EVENT_MANAGER:RegisterForEvent("ZAM_ReactionColors", EVENT_ADD_ON_LOADED, function(event, addon)
		if addon == "ZAM_ReactionColors" then
			Initialize()
		end
	end)
EVENT_MANAGER:RegisterForEvent("ZAM_ReactionColors", EVENT_LEVEL_UPDATE, function() playerLevel = GetUnitLevel("player") end)
