ZAMUF_UnitAttributeVisualizer = ZO_CallbackObject:Subclass()

function ZAMUF_UnitAttributeVisualizer:New(...)
    local unitAttributeVisualizer = ZO_CallbackObject.New(self)
    unitAttributeVisualizer:Initialize(...)
    return unitAttributeVisualizer
end

function ZAMUF_UnitAttributeVisualizer:Initialize(unitTag, healthBarControl, magickaBarControl, staminaBarControl, externalControlCallback)
    self.unitTag = unitTag

    self.visualModules = {}

    self.healthBarControl = healthBarControl
    self.magickaBarControl = magickaBarControl
    self.staminaBarControl = staminaBarControl

    self.moduleControlledCounts = {}
    if externalControlCallback then
        self.externalControlCallback = externalControlCallback
        if healthBarControl then
            self.moduleControlledCounts[healthBarControl] = 0
        end
        if magickaBarControl then
            self.moduleControlledCounts[magickaBarControl] = 0
        end
        if staminaBarControl then
            self.moduleControlledCounts[staminaBarControl] = 0
        end
    end

    local eventNamespace = "ZAMUF_UnitAttributeVisualizer" .. unitTag
    EVENT_MANAGER:RegisterForEvent(eventNamespace, EVENT_UNIT_ATTRIBUTE_VISUAL_ADDED, function(eventCode, ...) self:OnUnitAttributeVisualAdded(...) end)
    EVENT_MANAGER:RegisterForEvent(eventNamespace, EVENT_UNIT_ATTRIBUTE_VISUAL_UPDATED, function(eventCode, ...) self:OnUnitAttributeVisualUpdated(...) end)
    EVENT_MANAGER:RegisterForEvent(eventNamespace, EVENT_UNIT_ATTRIBUTE_VISUAL_REMOVED, function(eventCode, ...) self:OnUnitAttributeVisualRemoved(...) end)

    if unitTag == "reticleover" then
        local function OnReticleTargetChanged()
            self:OnUnitChanged()
        end
        EVENT_MANAGER:RegisterForEvent(eventNamespace, EVENT_RETICLE_TARGET_CHANGED, OnReticleTargetChanged)
    elseif unitTag == "target" then
        local function OnTargetChanged(evt, unitTag)
            if unitTag == "player" then
                self:OnUnitChanged()
            end
        end

        EVENT_MANAGER:RegisterForEvent(eventNamespace, EVENT_TARGET_CHANGED, OnTargetChanged)
    end
end

function ZAMUF_UnitAttributeVisualizer:OnUnitChanged()
    if DoesUnitExist(self.unitTag) then
        for module in pairs(self.visualModules) do
            module:OnUnitChanged()
        end
    end
end

function ZAMUF_UnitAttributeVisualizer:AddModule(module)
    if not self.visualModules[module] then
        module:SetOwner(self)
        module:OnAdded(self.healthBarControl, self.magickaBarControl, self.staminaBarControl)

        self.visualModules[module] = module
    end
end

function ZAMUF_UnitAttributeVisualizer:NotifyTakingControlOf(control)
    if self.moduleControlledCounts and self.moduleControlledCounts[control] then
        self.moduleControlledCounts[control] = self.moduleControlledCounts[control] + 1
        self.externalControlCallback(control, 1, self.moduleControlledCounts[control])
    end
end

function ZAMUF_UnitAttributeVisualizer:NotifyEndingControlOf(control)
    if self.moduleControlledCounts and self.moduleControlledCounts[control] then
        self.moduleControlledCounts[control] = self.moduleControlledCounts[control] - 1
        self.externalControlCallback(control, -1, self.moduleControlledCounts[control])
    end
end

function ZAMUF_UnitAttributeVisualizer:GetUnitTag()
    return self.unitTag
end

function ZAMUF_UnitAttributeVisualizer:OnUnitAttributeVisualAdded(unitTag, visualType, stat, attribute, powerType, value, maxValue)
    if self.unitTag == unitTag then
        for module in pairs(self.visualModules) do
            if module:IsUnitVisualRelevant(visualType, stat, attribute, powerType) then
                module:OnUnitAttributeVisualAdded(visualType, stat, attribute, powerType, value, maxValue)
            end
        end
    end
end

function ZAMUF_UnitAttributeVisualizer:OnUnitAttributeVisualUpdated(unitTag, visualType, stat, attribute, powerType, oldValue, newValue, oldMaxValue, newMaxValue)
    if self.unitTag == unitTag then
        for module in pairs(self.visualModules) do
            if module:IsUnitVisualRelevant(visualType, stat, attribute, powerType) then
                module:OnUnitAttributeVisualUpdated(visualType, stat, attribute, powerType, oldValue, newValue, oldMaxValue, newMaxValue)
            end
        end
    end
end

function ZAMUF_UnitAttributeVisualizer:OnUnitAttributeVisualRemoved(unitTag, visualType, stat, attribute, powerType, value, maxValue)
    if self.unitTag == unitTag then
        for module in pairs(self.visualModules) do
            if module:IsUnitVisualRelevant(visualType, stat, attribute, powerType) then
                module:OnUnitAttributeVisualRemoved(visualType, stat, attribute, powerType, value, maxValue)
            end
        end
    end
end
